/****************************************************************/
/*                                                              */
/*             Stage 4 filter for use with Weasel               */
/*                                                              */
/*   This filter gives special treatment to mail whose header   */
/*   contains the line "X-PostmasterCheck: FAIL".  That header  */
/*   line is inserted by Weasel if the domain in the MAIL FROM  */
/*   command does not have a "postmaster" account.  Since all   */
/*   legitimate mail servers MUST have a postmaster account     */
/*   (see RFC 822, for example), a failure of this test         */
/*   indicates either an invalid domain name, or a rogue mail   */
/*   server.  In either case, we suspect spam.                  */
/*                                                              */
/*   If we find the failure header, we move a copy of the       */
/*   message into the directory 'suspect', which will be a      */
/*   subdirectory of Weasel's working directory.  (This script  */
/*   creates the subdirectory if necessary.)  Apart from making */
/*   this copy, the message is allowed to be processed in the   */
/*   normal way.                                                */
/*                                                              */
/*   The reason for not taking tougher action is that the       */
/*   postmaster check is still experimental.  If tests reveal   */
/*   that this is a reliable spam check, and that it is not     */
/*   picking up non-spam messages, then a later version of this */
/*   script will probably take tougher action.                  */
/*                                                              */
/*      Author:       Peter Moylan (peter@pmoylan.org)          */
/*      Last revised: 2 December 2015                           */
/*                                                              */
/*   USAGE:                                                     */
/*      Use the Weasel Setup program to specify the name of     */
/*      this command file as a stage 4 filter.  If you already  */
/*      have another stage 4 filter, then look for the          */
/*      'multifilter' filter on my web site; this is a script   */
/*      to let you run multiple filters in sequence.            */
/*                                                              */
/*      Then check the 'suspect' directory periodically to      */
/*      see whether this is working as a spam check.  (It won't */
/*      pick up all spam, of course -- only one category of     */
/*      spam.)  If you find non-spam messages in the 'suspect'  */
/*      directory, please let me know, because that would       */
/*      indicate a serious problem.                             */
/*                                                              */
/****************************************************************/

PARSE ARG userlist SrcFile
SrcFile = STRIP(SrcFile)

/* Look for the specified header line. */

found = 0
deferred = 0
receivedcount = 0

DO FOREVER

    /* Each time around this loop we look at one line of   */
    /* the header.                                         */

    line = LineIn(SrcFile)
    IF line = "" THEN LEAVE        /* end of header */
    ELSE IF LEFT(line, 9) = "Received:" THEN
        DO
            /* A match after the second "Received:" doesn't count. */
            IF receivedcount > 0 THEN LEAVE
            receivedcount = 1
        END
    ELSE IF line = "X-PostmasterCheck: DEFERRED" THEN
        DO
            deferred = 1
            LEAVE
        END
    ELSE IF line = "X-PostmasterCheck: FAIL" THEN
        DO
            found = 1
            LEAVE
        END
END
CALL stream SrcFile, 'C', 'CLOSE'

IF deferred THEN
    DO
        /* Make 'deferred' directory if necessary. */

        '@IF NOT EXIST deferred MKDIR deferred'

        /* Copy the file. */

        pos = LASTPOS('\', SrcFile)
        dest = RIGHT(SrcFile, LENGTH(SrcFile)-pos)
        dest = "deferred\"dest
        '@COPY 'SrcFile' 'dest' >nul'
    END

ELSE IF found THEN
    DO
        /* Make 'suspect' directory if necessary. */

        '@IF NOT EXIST suspect MKDIR suspect'

        /* Copy the file. */

        pos = LASTPOS('\', SrcFile)
        dest = RIGHT(SrcFile, LENGTH(SrcFile)-pos)
        dest = "suspect\"dest
        '@COPY 'SrcFile' 'dest' >nul'
    END

/* In this version, return an "OK" code regardless of the result. */

EXIT 0

